// pcr1000.h: interface for the pcr1000 class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __PCR1000_H__
#define __PCR1000_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "terminalIO.h"

typedef unsigned char Value;

class pcr1000  
{
public:
	pcr1000(const char *const ttyname);
	virtual ~pcr1000();

	/* accessors */
	bool anf(void) const;
	bool autoUpdate(void) const;
	bool bandscopeActive(void) const;
	bool commandStatus(void) const;
	bool dspON(void) const;
	bool isAGCOn(void) const;
	bool isAttenuationOn(void) const;
	bool isDSPInstalled(void) const;
	bool isLinearPowerOn(void) const;
	bool isNBOn(void) const;
	bool isVSCOn(void) const;
	bool updateAscii(void) const;

	Value bandscopeSamplingWidth(void) const;
	Value bandscopeSamplingRate(void) const;
	Value bfo(void) const;
	Value DTMFTone(void) const;
	Value IFShift(void) const;
	Value noiseReductionLevel(void) const;
	Value signalStrength(void) const;
	Value signalCentering(void) const;
	Value squelch(void) const;
	Value toneSquelch(void) const;
	Value volume(void) const;

	const Value * const samples(int window) const;

	const char * const protocol(void) const;
	const char * const regionCode(void) const;

	double bandscopeCenterFrequency(void) const;
	double frequency(void) const;

	int baudRate(void) const;
	int decodeMode(void) const;
	int filterMode(void) const;
	int scanStatus(void) const;

	/* modifiers */
	void AGC(bool newMode);
	void anf(bool onOff);
	void attenuation(bool newMode);
	void autoUpdate(bool newMode);
	void baudRate(int rate);
	void bfo(Value newBFO);
	void bandscope(bool onOff);
	void bandscopeCenterFrequency(double newFrequency);
	void bandscopeSamplingRate(Value newSamplingRate);
	void bandscopeSamplingWidth(Value newSamplingWidth);
	void decodeMode(int newMode);
	void dsp(bool newMode);
	void filterWidth(int newFilterWidth);
	void frequency(double newFrequency);
	void IFShift(Value newIFShift);
	void linearPower(bool onOff);
	void NB(bool newMode);
	void noiseReductionLevel(Value newLevel);
	void poll(bool onOff);
	void pollAscii(bool onOff);
	void reset(void);
	void squelch(Value newSquelch);
	void toneSquelch(Value newToneSquelch);
	void volume(Value newVolume);
	void VSC(bool newMode);

private:
	/* We only support one pcr1000 access at a time*/
	pcr1000(const pcr1000&); /* no public copy constructor */
	pcr1000& operator=(const pcr1000&); /* and no assignment */

	/* Utility functions */
	void init_radio(void);

	terminalIO *m_tty;	/* name of device file for tty */

	bool m_autoUpdate;	/* true if pcr is sending updates, false:poll */
	bool m_AGC;		/* automatic gain control, true if on */
	bool m_Attenuation;	/* Attenuator, true if on */
	bool m_bandscope;	/* true if bandscope on */
	bool m_DSPInstalled;	/* true if the optional DSP unit is present */
	bool m_NB;		/* noise blanker, true if on */
	bool m_poll;
	bool m_pollMode;	/* true = ascii, false = binary */
	bool m_power;	/* true if linear power is on */
	bool m_status;	/* status of last command sent to pcr */
	bool m_squelchStatus;
	bool m_VSC;		/* voice squelch, true if on */

	Value m_BFO;		/* bfo offset in range 1-255 */
	Value m_DTMFTone;
	Value m_IFShift;		/* ifshift in range 1-255 */
	Value m_squelch;		/* squelch in range 1-255 */
	Value m_volume;		/* volume in range 1-255 */
	Value m_SamplingRate;	/* encoded bandscope sampling rate (ms) */
	Value m_SamplingWidth;	/* width of one bandscope sample */
	Value m_signalCentering;
	Value m_signalStrength;
	Value m_toneSquelch;	/* ctcss is in use */

	char *m_regionCode;

	double m_CenterFrequency; /* bandscope center frequency */
	double m_frequency;	/* last tuned frequency */

	int m_decodeMode;	/* last set decode mode */
	int m_filterMode;	/* last set filter width */

	unsigned int m_baudRate; /* the baud rate that the tty is set to */
};

#endif // __PCR1000_H__

