#ifndef __PCRDEFS_H__
#define __PCRDEFS_H__

#define TYPE_ONOFF 1		/* command is binary */
#define TYPE_RANGE 2		/* command takes an arg in 0-255 */
#define TYPE_LIST  3		/* command takes a list arg */
#define TYPE_FREQ  4		/* command takes 0.050 - 1300.0 */
#define TYPE_BAND  5		/* command is bandscope (blah) */
#define TYPE_QUERY 6		/* command is a query */

#define COMMAND_COMMAND_STATUS  "G0?"
#define COMMAND_BAUD            "G1xx"
#define COMMAND_PROTOCOL        "G2?"
#define COMMAND_AUTOUPDATE      "G30x"
#define COMMAND_DSP_PRESENT     "GD?"
#define COMMAND_REGION_CODE     "GE?"
#define COMMAND_RESET           "H001"
#define COMMAND_POWER_ONOFF     "H1?"
#define COMMAND_ONOFF           "H10x"
#define COMMAND_SCAN_MODE       "H8xx"
#define COMMAND_SCAN_STATUS     "H9?"
#define COMMAND_STATUS          "I0?"
#define COMMAND_STRENGTH        "I1?"
#define COMMAND_CENTER          "I2?"
#define COMMAND_DTMF_TONE       "I3?"
#define COMMAND_VOLUME          "J40xx"
#define COMMAND_SQUELCH         "J41xx"
#define COMMAND_IFSHIFT         "J43xx"
#define COMMAND_AGC             "J450x"
#define COMMAND_NOISEBLANKER    "J460x"
#define COMMAND_ATTENUATOR      "J470x"
#define COMMAND_BFO             "J4Axx"
#define COMMAND_VSC             "J500x"
#define COMMAND_TONESQUELCH_OFF "J5100"
#define COMMAND_TONESQUELCH_SET "J51xx"
#define COMMAND_DSP_PREFIX      "J8001"
#define COMMAND_DSP_ONOFF       "J81xx"
#define COMMAND_DSP_NR          "J82xx"
#define COMMAND_DSP_ANF         "J83xx"
#define COMMAND_FREQUENCY       "K0gmmmkkkhhhmmff00"
#define COMMAND_TRACKINGFILTER  "LD82000"
#define COMMAND_BANDSCOPE       "ME00001ssrrmm00ffffff"
#define COMMAND_GET_BANDSCOPE   "NE0xx0"

#define COMMAND_ON              "1"
#define COMMAND_OFF             "0"

#define BAUD_RATE_1200          "00"
#define BAUD_RATE_2400          "01"
#define BAUD_RATE_4800          "02"
#define BAUD_RATE_9600          "03"
#define BAUD_RATE_19200         "04"
#define BAUD_RATE_38400         "05"


#define DSP_BIT_DSP               0
#define DSP_BIT_DARC              4

#define SCAN_MODE_EDGE            0
#define SCAN_MODE_LEVEL           0
#define SCAN_MODE_MASK           0x40 // bit 7 on = level off = edge
#define SCAN_MASK_TIME           0x3F // low order 6 bits

#define SCAN_STATUS_NOT         "00"
#define SCAN_STATUS_READY       "01"
#define SCAN_STATUS_REVIEW      "02"
#define SCAN_STATUS_SQUELCH     "03"
#define SCAN_STATUS_TONE        "04"
#define SCAN_STATUS_VSC         "05"
#define SCAN_STATUS_BUSY        "10"

#define FREQUENCY_MODE_LSB      "00"
#define FREQUENCY_MODE_USB      "01"
#define FREQUENCY_MODE_AM       "02"
#define FREQUENCY_MODE_CW       "03"
#define FREQUENCY_MODE_NA       "04"
#define FREQUENCY_MODE_NFM      "05"
#define FREQUENCY_MODE_WFM      "06"

#define FILTER_WIDTH_2K8        "00" /* (CW USB LSB AM) */
#define FILTER_WIDTH_6K         "01" /* (CW USB LSB AM NFM) */
#define FILTER_WIDTH_15K        "02" /* (AM NFM) */
#define FILTER_WIDTH_50K        "03" /* (AM NFM WFM) */
#define FILTER_WIDTH_230K       "04" /* (WFM) */

#define STATUS_GOOD             "00"
#define STATUS_BAD              "01"

#define SIGNAL_MODE_POLL        "00"
#define SIGNAL_MODE_PUSH        "01"
#define SIGNAL_MODE_BINARY_POLL "02"
#define SIGNAL_MODE_BINARY_PUSH "03"

#define SQUELCH_CLOSED          "04"
#define SQUELCH_OPEN            "07"

#define FREQUENCY_LOW           "00" /* signal frequency < display frequency */
#define FREQUENCY_EQUAL         "80" /* signal frequency == display frequency */
#define FREQUENCY_HIGH          "FF" /* signal frequency > display frequency */

#define DTMF_OFF                "00"
#define DTMF_TONE               "1?"

#define QUERY_FIRMWARE          "G4"
#define QUERY_REGION            "GE"
#define QUERY_DSPINSTALLED      "GD"
#define QUERY_ONOFF             "H1"
#define QUERY_SQUELCH           "I0"
#define QUERY_SIGNAL_STRENGTH   "I1"
#define QUERY_SIGNAL_CENTERING  "I2"

#define BANDSCOPE_PACKET_LENGTH 37
#define BANDSCOPE_PACKET_FORMAT "NE1ppaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"

#endif /* __PCRDEFS_H__ */
