
static int is_set(tcflag_t flag, tcflag_t bit)
{
  return ((flag & bit) == bit);
}

static void print_ldisc(const char * const tty_name, int ldisc)
{
  fprintf(stdout,"Line discipline of \"%s\" is ", tty_name);
  switch(ldisc) {
  case TTYDISC:
    fprintf(stdout,"termios interactive.\n");
    break;
  case TABLDISC:
    fprintf(stdout,"tablet.\n");
    break;
  case SLIPDISC:
    fprintf(stdout,"Serial IP.\n");
    break;
  case PPPDISC:
    fprintf(stdout,"PPP.\n");
    break;
  default:
    fprintf(stdout,"unknown (%d)\n", ldisc);
  }
}

static void print_iflag(tcflag_t flag)
{
  fprintf(stdout, "I flag = 0x%x\n", flag);
  if (is_set(flag, IGNBRK))
    fprintf(stdout,"\tignore BREAK\n");
  if (is_set(flag, BRKINT))
    fprintf(stdout,"\tmap BREAK to SIGINTR\n");
  if (is_set(flag,IGNPAR ))
    fprintf(stdout,"\tignore parity errors\n");
  if (is_set(flag, PARMRK))
    fprintf(stdout,"\tmark parity\n");
  if (is_set(flag, INPCK))
    fprintf(stdout,"\tenable parity error checking\n");
  if (is_set(flag, ISTRIP))
    fprintf(stdout,"\tstrip 8th bit\n");
  if (is_set(flag, INLCR))
    fprintf(stdout,"\tmap NL to CR\n");
  if (is_set(flag, IGNCR))
    fprintf(stdout,"\tignore CR\n");
  if (is_set(flag, ICRNL))
    fprintf(stdout,"\tmap CR to NL\n");
  if (is_set(flag, IXON))
    fprintf(stdout,"\tenable output flow control\n");
  if (is_set(flag, IXOFF))
    fprintf(stdout,"\tenable input flow control\n");
#ifndef _POSIX_SOURCE
  if (is_set(flag, IXANY))
    fprintf(stdout,"\tany char restart after stop\n");
  if (is_set(flag, IMAXBEL))
#endif /*_POSIX_SOURCE */
    fprintf(stdout,"\tbell on input queue full\n");
#if 0
  if (is_set(flag, IUCLC))
    fprintf(stdout,"\txlate upper to lower\n");
#endif
}

static void print_oflag(tcflag_t flag)
{
  fprintf(stdout, "O flag = 0x%x\n", flag);
  if (is_set(flag, OPOST))
    fprintf(stdout,"\tenable following output processing\n");
#ifndef _POSIX_SOURCE
  if (is_set(flag, ONLCR))
    fprintf(stdout,"\tmap NL to CR-NL\n");
  if (is_set(flag, OXTABS))
    fprintf(stdout,"\texpand tabs to space\n");
  if (is_set(flag, ONOEOT))
    fprintf(stdout,"\tdiscard EOTs on output\n");
#endif
#if 0
  if (is_set(flag, OCRNL))
    fprintf(stdout,"\tmap CR to NL\n");
  if (is_set(flag, OLCUC))
    fprintf(stdout,"\txlate lower cast to upper\n");
  if (is_set(flag, ONOCR))
    fprintf(stdout,"\tNo CR in column 0\n");
  if (is_set(flag, ONLRET))
    fprintf(stdout,"\tNL performs CR\n");
#endif
  fprintf(stdout,"\n");
}

static void print_cflag(tcflag_t flag)
{
  fprintf(stdout, "C flag = 0x%x\n", flag);
#ifndef _POSIX_SOURCE
  if (is_set(flag, CIGNORE))
    fprintf(stdout,"\tignore control flags\n");
#endif
  if (is_set(flag, CSIZE)) {
    fprintf(stdout,"\tChar size set to\n");
    if (is_set(flag, CS8)) {
      fprintf(stdout,"\t\t 8 bits\n");
    } else if (is_set(flag, CS7)) {
      fprintf(stdout,"\t\t 7 bits\n");
    } else if (is_set(flag, CS6)) {
      fprintf(stdout,"\t\t 6 bits\n");
    } else if (is_set(flag, CS5)) {
      fprintf(stdout,"\t\t 5 bits (pseudo)\n");
    } else {
      fprintf(stdout,"\t\tCant decode size\n");
    }
  }
  if (is_set(flag, CSTOPB))
    fprintf(stdout,"\tsend 2 stop bits\n");
  if (is_set(flag, CREAD))
    fprintf(stdout,"\tenable receiver\n");
  if (is_set(flag, PARENB))
    fprintf(stdout,"\tparity enable\n");
  if (is_set(flag, PARODD))
    fprintf(stdout,"\tparity odd\n");
  else
    fprintf(stdout,"\tparity even\n");
  if (is_set(flag, HUPCL))
    fprintf(stdout,"\thang up on last close\n");
  if (is_set(flag, CLOCAL))
    fprintf(stdout,"\tignore modem status lines\n");
#ifndef _POSIX_SOURCE
  if (is_set(flag, CCTS_OFLOW))
    fprintf(stdout,"\tuse CTS flow control\n");
  if (is_set(flag, CRTSCTS))
    fprintf(stdout,"\tCRTSCTS\n");
  if (is_set(flag, CRTS_IFLOW))
    fprintf(stdout,"\tRTS flow control of input\n");
  if (is_set(flag, CDTR_IFLOW))
    fprintf(stdout,"\tDTR flow control of input\n");
  if (is_set(flag, CDSR_OFLOW))
    fprintf(stdout,"\tDSR flow control of output\n");
  if (is_set(flag, CCAR_OFLOW))
    fprintf(stdout,"\tCCAR flow control of output\n");
  if (is_set(flag, MDMBUF))
    fprintf(stdout,"\tCCAR flow control of output\n");
#endif
  fprintf(stdout,"\n");
}

static void print_lflag(tcflag_t flag)
{
  fprintf(stdout, "L flag = 0x%x", flag);
#ifndef _POSIX_SOURCE
  if (is_set(flag, ECHOKE))
    fprintf(stdout,"\tvisual erase for line kill\n");
#endif
  if (is_set(flag, ECHOE))
    fprintf(stdout,"\tvisually erase chars\n");
  if (is_set(flag, ECHO))
    fprintf(stdout,"\tenable echoing\n");
  if (is_set(flag, ECHONL))
    fprintf(stdout,"\techo nL even if ECHO is off\n");
  if (is_set(flag, ECHOPRT))
    fprintf(stdout,"\tvisual erase hardcopy\n");
  if (is_set(flag, ECHOCTL))
    fprintf(stdout,"\techo ctrl as ^(ch)\n");
  if (is_set(flag, ISIG))
    fprintf(stdout,"\tenable INTR, QUIT, [D]SUSP\n");
  if (is_set(flag, ICANON))
    fprintf(stdout,"\tcanonicalize input\n");
#ifndef _POSIX_SOURCE
  if (is_set(flag, ALTWERASE))
    fprintf(stdout,"\tuse alternate WERASE algorithm\n");
#endif
  if (is_set(flag, IEXTEN))
    fprintf(stdout,"\tenable DISCARD and LNEXT\n");
  if (is_set(flag, EXTPROC))
    fprintf(stdout,"\texternal processing\n");
  if (is_set(flag, TOSTOP))
    fprintf(stdout,"\tstop background jobs from output\n");
#ifndef _POSIX_SOURCE
  if (is_set(flag, FLUSHO))
    fprintf(stdout,"\toutput being flushed (state)\n");
  if (is_set(flag, NOKERNINFO))
    fprintf(stdout,"\tno kernel output from VSTATUS\n");
  if (is_set(flag, PENDIN))
    fprintf(stdout,"\tXXX retype pending input (state)\n");
#endif
  if (is_set(flag, NOFLSH))
    fprintf(stdout,"\tdon't flush after interrupt\n");
  fprintf(stdout,"\n");
}

static char *cc_names[NCCS] = {
  "EOF", "EOL", "EOL2", "ERASE", "WERASE", "KILL",
  "REPRINT", "SPARE", "INTR", "QUIT", "SUSP", "DSUSP",
  "START", "STOP", "LNEXT", "DISCARD", "VMIN",
  "VTIME", "STATUS", "SPARE"
};

static void print_cc(cc_t *cc)
{
  unsigned char x;
  int i;
  for (i = 0; i < NCCS; i++) {
    fprintf(stdout,"cc[%s](0x%x) = ", cc_names[i], cc[i]);
    if (isprint(cc[i])) {
      fprintf(stdout,"%c", cc[i]);
    } else if (cc[i] == 0xff) {
      fprintf(stdout,"undefined");
    } else if (cc[i] == 0x7f) {
      fprintf(stdout,"<DEL>");
    } else if (iscntrl(cc[i])) {
      fprintf(stdout,"^%c", cc[i] + 'A' - 1);
    } else {
      fprintf(stdout, "(0x%x)", cc[i]);
    }
    fprintf(stdout,"\n");
  }
}


static int debug_put_termios(const char * const tty_name, int tty,
		       struct termios const * const src_term)
{
  int i;
  struct termios put_term = *src_term;
  struct termios get_term;
  if (ioctl(tty, TIOCSETA, &put_term) < 0) {
    fprintf(stderr,"Setting termios failed on \"%s\"\n", tty_name);
    perror("ioctl");
    return -1;
  }
  if (ioctl(tty, TIOCGETA, &get_term) < 0) {
    fprintf(stderr,"Getting termios (to verify) failed on \"%s\"\n", tty_name);
    perror("ioctl");
    return -1;
  }
  if (put_term.c_iflag != get_term.c_iflag) {
    fprintf(stderr,"Set failed. Tried to set iflag to 0x%x, but set to 0x%x\n",
	    put_term.c_iflag, get_term.c_iflag);
    return -1;
  }
  if (put_term.c_oflag != get_term.c_oflag) {
    fprintf(stderr,"Set failed. Tried to set oflag to 0x%x, but set to 0x%x\n",
	    put_term.c_oflag, get_term.c_oflag);
    return -1;
  }
  if (put_term.c_cflag != get_term.c_cflag) {
    fprintf(stderr,"Set failed. Tried to set cflag to 0x%x, but set to 0x%x\n",
	    put_term.c_cflag, get_term.c_cflag);
    return -1;
  }
  if (put_term.c_lflag != get_term.c_lflag) {
    fprintf(stderr,"Set failed. Tried to set lflag to 0x%x, but set to 0x%x\n",
	    put_term.c_lflag, get_term.c_lflag);
    return -1;
  }
  for (i = 0; i < NCCS; i++) {
    if (put_term.c_cc[i] != get_term.c_cc[i]) {
      fprintf(stderr,"Set failed. Tried to set c_cc[%d] to 0x%x, but set to 0x%x\n",
	      i, put_term.c_cc[i], get_term.c_cc[i]);
      return -1;
    }
  }
  if (put_term.c_ispeed != get_term.c_ispeed) {
    fprintf(stderr,"Set failed. Tried to set ispeed to 0x%x, but set to 0x%x\n",
	    put_term.c_ispeed, get_term.c_ispeed);
    return -1;
  }
  if (put_term.c_ospeed != get_term.c_ospeed) {
    fprintf(stderr,"Set failed. Tried to set ospeed to 0x%x, but set to 0x%x\n",
	    put_term.c_ospeed, get_term.c_ospeed);
    return -1;
  }
  return 0;
}
